#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging
from PyQt4 import QtGui, QtCore
from dbus_manager import Nfssync_dbus
from pybytes import bytes2human
from threading import Thread, Event

logging.basicConfig(level=logging.DEBUG)

class SysTrayIcon(QtGui.QWidget):
    animate_icon_signal = QtCore.pyqtSignal()

    def __init__(self, parent=None):
        super(SysTrayIcon, self).__init__(parent)

        self.dbus = Nfssync_dbus()
        
        self.register_handlers()
        self.register_msgs()

        self.quota_total = 0.0
        self.quota_used = 0.0
        self.last_progress = 100
        self.animate_dir = True
        self.animate_index = 1

        self.show_quota_action = QtGui.QAction("Conectando con el servidor...",     self, triggered=self.updateUI)

        self.update_action     = QtGui.QAction("Actualizar ahora",     self, triggered=self.update_rsync)
        #self.quit_action       = QtGui.QAction("Quit (esto se quita)",        self, triggered=QtGui.qApp.quit)

        self.dbus.emit_get_quota_signal()

        self.createSystrayIcon()
        self.systrayIcon.show()

    def createSystrayIcon(self):
        self.systrayIconMenu = QtGui.QMenu(self)

        self.systrayIconMenu.addAction(self.show_quota_action)
        self.systrayIconMenu.addSeparator()
        self.systrayIconMenu.addAction(self.update_action)
        
        #self.systrayIconMenu.addSeparator()
        #self.systrayIconMenu.addAction(self.quit_action)

        self.systrayIcon = QtGui.QSystemTrayIcon(self)
        self.systrayIcon.setContextMenu(self.systrayIconMenu)

        self.systrayIcon.setVisible(True)

    def register_handlers(self):
        self.dbus.register_user_quota_signal_handler(self.handler_quota)
        self.dbus.register_rsync_status_signal_handler(self.handler_status)
        self.dbus.register_rsync_prog_signal_handler(self.handler_prog)
        self.animate_icon_signal.connect(self.animate_icon)

    def register_msgs(self):
        self.notify_init = lambda: notifyMsg(title="Iniciando sincronización")
        self.notify_ok = lambda: notifyMsg(title="Sincronización completada")
        self.notify_error = lambda: notifyMsg(title="Error en la sincronización", msg="Es posible que necesite liberar espacio.", dialog="dialog-error")

    def handler_quota(self, total, used):
        logging.debug("Cuota recibida (Kbytes): %s/%s" % (total, used))
        self.quota_total = total #in Kbytes
        self.quota_used = used #in Kbytes
        self.updateUI()

    def handler_status(self, status):
        logging.debug("Status recibido: %d" % status)
        self.update_action.setText("Actualizar ahora")
        self.update_action.setEnabled(True)
        self.animation.stop()
        self.animate_dir = True
        self.animate_index = 1
        
        if self.last_progress < 100:
            logging.debug("No se han sincronizado todos los archivos!!! Necesita liberar espacio.")
            self.notify_error()
        else:
            logging.debug("Sincronización terminada con éxito..")
            self.notify_ok()
        
        self.dbus.emit_get_quota_signal()

    def handler_prog(self, progress):
        logging.debug("Progress recibido: %d" % progress)
        self.update_action.setText("Actualizando... (%d%%)" % (progress))
        self.last_progress = progress

    def update_rsync(self):
        self.dbus.emit_rsync_signal(True)
        self.notify_init()
        self.update_action.setDisabled(True)
        self.update_action.setText("Actualizando... (0%)")
        self.animation = PeriodicSignal(0.12, self.animate_icon_signal)
        self.animation.start()
    
    def animate_icon(self):
        if self.animate_index < 10:
            self.animate_index = self.animate_index + 1
        else:
            self.animate_index = 1
        
        self.systrayIcon.setIcon(QtGui.QIcon("icons/waiting/iconw%d.png" % self.animate_index))

    def updateUI(self):
        if self.quota_total > 0.0:
            porc_usado = (self.quota_used * 100) / self.quota_total
            total = bytes2human(self.quota_total*1024)
            usado = bytes2human(self.quota_used*1024)

            self.show_quota_action.setText("Usado %.0f%% (%s de %s)" % (porc_usado, usado, total))

            #El Icono...
            if porc_usado > 90 or self.last_progress < 100: # (last_progress < 100) => Sincronización incompleta!
                icon_index = 11
            else:
                icon_index = int(round(porc_usado/10, 0))
                if icon_index == 0:
                        icon_index = 1
            self.systrayIcon.setIcon(QtGui.QIcon("icons/icon_%d.png" % icon_index))
        else:
            self.show_quota_action.setText("error conectando con el servidor")
            self.systrayIcon.setIcon(QtGui.QIcon("icons/icon_11.png"))


class notifyMsg(object):
        def __init__(self, title="", msg="", dialog="dialog-info", parent=None):
                import pynotify
                pynotify.init('app')
                n = pynotify.Notification(title, msg, dialog) # Titulo, Mensaje...
                n.show()

                
class PeriodicSignal(Thread):
    
    def __init__(self, interval, signal):
        Thread.__init__(self)
        self.interval = interval
        self.signal = signal
        self.finished = Event()
 
    ## CUIDADO!! Hay que invocar start(), no run() !!
    #
    def run(self):
        while not self.finished.is_set():
            self.finished.wait(self.interval)
            if not self.finished.is_set():
                self.signal.emit()

    def stop(self):
        self.finished.set()  
              
if __name__ == '__main__':
    app = QtGui.QApplication([])
    systrayicon = SysTrayIcon()
    app.exec_()

