#!/usr/bin/env python
# -*- coding: UTF-8 -*-
 
import logging
import gobject
import dbus.mainloop, dbus.service
from dbus.mainloop.glib import DBusGMainLoop

logging.basicConfig(level=logging.DEBUG)

class Nfssync_dbus(dbus.service.Object):
    def __init__(self):
        DBusGMainLoop(set_as_default=True)
        self.__bus = dbus.SystemBus()
        dbus.service.Object.__init__(self, 
                                     self.__bus, 
                                     "/cga/nfssyncd/dbus/signal")
        
        self.__RSYNC_STATUS = "RSYNC_STATUS"
        self.__RSYNC_PROG   = "RSYNC_PROG"
        self.__USER_QUOTA   = "USER_QUOTA"
        
        self.__signals = [self.__RSYNC_PROG,
                          self.__RSYNC_STATUS,
                          self.__USER_QUOTA] 
        
        self.__signal_handlers={ self.__RSYNC_STATUS: None,
                                self.__RSYNC_PROG  : None,
                                self.__USER_QUOTA   : None}
        
        self.__init_receivers()
    
    ## Emite la señal RSYNC
    #@param on_progress: Boolean 
    @dbus.service.signal(dbus_interface='cga.nfssyncd.dbus.Signal',
                         signature='b')
    def RSYNC(self, on_progress):
        logging.debug( "Se emite RSYNC<%s>" %(on_progress))
        
    
    # Emite la señal RSINC
    #@param on_progress: Boolean
    def emit_rsync_signal(self, on_progress):
        self.RSYNC(on_progress)
    
    ## Emite la señal SETUP_USER
    #
    #TODO: parámetros
    @dbus.service.signal(dbus_interface='cga.nfssyncd.dbus.Signal',
                         signature='')
    def GET_QUOTA(self):
        print "Se emite GET_CUOTA<void>"
    
    
    def emit_get_quota_signal(self):
        self.GET_QUOTA()
    
    ############################################################
    ##  
    ##    HANDLING NFSSYNCD SIGNALS
    ## 
    
    ##
    #  signal RSYNC_PROG (int32,)
    def __private_rsync_prog_receiver(self, int_arg):
        logging.debug("RSYNC_PROG signal received")
        handler = self.__signal_handlers[self.__RSYNC_PROG]
        if handler and hasattr(handler, "__call__"):
            handler(int_arg)
    
    ##
    #  signal RSYNC_STATUS (boolean,)
    def __private_rsync_status_receiver(self, bool_arg):
        logging.debug("RSYNC_STATUS signal received")
        handler = self.__signal_handlers[self.__RSYNC_STATUS]
        if handler and hasattr(handler, "__call__"):
            handler(bool_arg)
    
    ##
    #  signal USER_QUOTA(int32, int32,)
    def __private_rsync_user_quota_receiver(self, int_arg_1, int_arg_2):
        logging.debug("USER_QUOTA(%s,%s) signal received" %(int_arg_1, int_arg_2))
        handler = self.__signal_handlers[self.__USER_QUOTA]
        if handler and hasattr(handler, "__call__"):
            handler(int_arg_1,int_arg_2)
    
    def __init_receivers(self):
        self.__add_receiver(self.__RSYNC_PROG, self.__private_rsync_prog_receiver)
        self.__add_receiver(self.__RSYNC_STATUS, self.__private_rsync_status_receiver)
        self.__add_receiver(self.__USER_QUOTA, self.__private_rsync_user_quota_receiver)
            
    
    def __add_receiver(self,signal,handler):
        signal_math = self.__bus.add_signal_receiver(handler, 
                                                     signal, 
                                                     dbus_interface='cga.nfssyncd.dbus.Signal')
        if not signal_math:
            logging.error("Impossible to register signal receiver.")
    
    
    def register_handler(self, signal, handler):
        # is callable handler param?
        if not handler or not hasattr(handler, "__call__"):
            raise Exception("Not callable handler parameter")
        # is signal in signals?
        if not signal in self.__signals:
            raise Exception("Non valid signal name")
        else:
            # add to private dictionary
            self.__signal_handlers[signal]=handler
        
        
    def register_rsync_status_signal_handler(self, handler):
        self.register_handler(self.__RSYNC_STATUS, handler)
        
    def register_rsync_prog_signal_handler(self, handler):
        self.register_handler(self.__RSYNC_PROG, handler)
        
    def register_user_quota_signal_handler(self, handler):
        self.register_handler(self.__USER_QUOTA, handler)


if __name__ == "__main__":
    
    logging.debug("Depurandooo")
    
    loop = gobject.MainLoop()
    gobject.threads_init()
    
    nfssync = Nfssync_dbus()
    
    nfssync.RSYNC(False)
    nfssync.GET_QUOTA()
    
    loop.run()
